#include "common.hpp"

__global__ void Multiply(const int *input, 
  int *output, const int multiplier, 
  const int width) {
  
  int tid = blockIdx.x;
  if(tid < width) 
    output[tid] = input[tid] * multiplier;
}

void PrintArray(const char* arrayName, 
  const int * arrayData, const int length);

const int Multiplier = 2;
const int Width = 128;
const int SizeInBytes = Width * sizeof(int);
int *Input = NULL, *Output = NULL;

int main() {
  Input = (int*)malloc(SizeInBytes);
  CHECK_NOT_NULL(Input);
  Output = (int*)malloc(SizeInBytes);
  CHECK_NOT_NULL(Output);
  for(int i = 0; i < Width; i++) Input[i] = i;
  PrintArray("Input", Input, Width);

  int *dev_input, *dev_output;
  CHECK_ERROR(
    cudaMalloc((void**)&dev_input, SizeInBytes));
  CHECK_ERROR(
    cudaMalloc((void**)&dev_output, SizeInBytes));
  CHECK_ERROR(
    cudaMemcpy(dev_input, Input, SizeInBytes, 
    cudaMemcpyHostToDevice));

  Multiply<<<Width, 1>>>(dev_input, dev_output, 
    Multiplier, Width);

  CHECK_ERROR(
    cudaMemcpy(Output, dev_output, SizeInBytes, 
    cudaMemcpyDeviceToHost));
  PrintArray("Output", Output, Width);

  CHECK_ERROR(cudaFree(dev_input));
  CHECK_ERROR(cudaFree(dev_output));
  free(Input);
  free(Output);
  return 0;
}

void PrintArray(const char* arrayName, 
  const int *arrayData, const int length) {
  
  cout << arrayName << ":" << endl;
  for(int i = 0; i < length; i++) 
    cout << arrayData[i] << " ";
  cout << endl << endl;
}
