#include "common.hpp"
#include <Windows.h>

__global__ void MakeSum(const unsigned int *input, 
  unsigned __int64 *output, const unsigned int width) {
  
  int tid = blockDim.x * blockIdx.x + threadIdx.x;
  unsigned __int64 sum = 0;
  if((tid > 0) && (tid < width)) {
    unsigned int min = input[tid - 1];
    unsigned int max = input[tid];
    unsigned int forAdd = min;
    while (forAdd < max) sum += forAdd++;
  }
  output[tid] = sum;
}

/*
void PrintArray(const char *arrayName, 
  const unsigned int *arrayData, 
  const unsigned int length);
void PrintArray64(const char *arrayName, 
  const unsigned __int64 *arrayData, 
  const unsigned int length);
*/

const unsigned int Min = 0;
const unsigned int BlockAndThread = 128;
const unsigned int Width = 
  BlockAndThread * BlockAndThread;
const unsigned int SizeInBytes = 
  Width * sizeof(unsigned int);
const unsigned int SizeInBytes64 = 
  Width * sizeof(unsigned __int64);
unsigned int *Input = NULL;
unsigned __int64 *Output = NULL;

int main(int argc, char *argv[]) {
  __int64 elapsedTime;
  __int64 startTime = GetTickCount();

  if(argc != 2) {
    cout << "1 arg is needed.\n";
    return 1;
  }
  unsigned int max = _atoi64(argv[1]);

  Input = (unsigned int*)malloc(SizeInBytes);
  CHECK_NOT_NULL(Input);
  Output = (unsigned __int64*)malloc(SizeInBytes64);
  CHECK_NOT_NULL(Output);
  
  unsigned int range = (max - Min) / (Width - 1);
  unsigned int midValue = Min + range;
  Input[0] = Min;
  Input[Width - 1] = max;
    for(unsigned int i = 1; i < Width - 1; i++) {
    Input[i] = midValue;
    midValue += range;
  }
  //PrintArray("Input", Input, Width);
  
  unsigned int *dev_input;
  unsigned __int64 *dev_output;
  CHECK_ERROR(
    cudaMalloc((void**)&dev_input, SizeInBytes));
  CHECK_ERROR(
    cudaMalloc((void**)&dev_output, SizeInBytes64));
  CHECK_ERROR(
    cudaMemcpy(dev_input, Input, SizeInBytes, 
    cudaMemcpyHostToDevice));

  MakeSum<<<BlockAndThread, BlockAndThread>>>(
    dev_input, dev_output, Width);
  
  CHECK_ERROR(cudaMemcpy(Output, dev_output, 
    SizeInBytes64, cudaMemcpyDeviceToHost));
  //PrintArray64("Output", Output, Width);

  unsigned __int64 sum = 0;
  for(unsigned int i = 1; i < Width; i++)
    sum += Output[i];
  sum += max;

  CHECK_ERROR(cudaFree(dev_input));
  CHECK_ERROR(cudaFree(dev_output));
  free(Input);
  free(Output);

  elapsedTime = GetTickCount() - startTime;
  cout << Min << '\t' << max << '\t' << 
    sum << '\t' << elapsedTime << endl;
  return 0;
}

/*
void PrintArray(const char *arrayName, 
  const unsigned int *arrayData, 
  const unsigned int length) {
  cout << arrayName << ":" << endl;
  for(int i = 0; i < length; i++) 
    cout << arrayData[i] << " ";
  cout << endl << endl;
}

void PrintArray64(const char *arrayName, 
  const unsigned __int64 *arrayData, 
  const unsigned int length) {
  
  cout << arrayName << ":" << endl;
  for(int i = 0; i < length; i++) 
    cout << arrayData[i] << " ";
  cout << endl << endl;
}
*/
