#include "common.hpp"
#include <Windows.h>
#include <math.h>

__global__ void GetArea(const double *input, 
  double *output, const unsigned int width, 
  const unsigned int divide) {
  
  const int tid = blockDim.x * blockIdx.x + threadIdx.x;
  if((tid > 0) && (tid < width)) {
    const double min = input[tid - 1];
    const double max = input[tid];
    const double dividedX = 
      (max - min) / (double)divide;
    double x = min + dividedX / 2;
    double y;
    double area = 0.0;

    for(unsigned int i = 0; i < divide; i++) {
      y = -0.006 * (x * x * x) + 0.1 * (x * x) + 
        20.0 * x + 1000.0;
      area += fabs(y) * dividedX;
      x += dividedX;
    }
    output[tid] = area;
  }
}

/*
void PrintArray(const string arrayName, 
  const double *arrayData, const unsigned int length);
*/

const double Min = 0.0;
const double Max = 50.0;
const unsigned int BlockAndThread = 100;
const unsigned int Width = 
  BlockAndThread * BlockAndThread;
const unsigned int SizeInBytes = 
  Width * sizeof(double);
double *Input = NULL;
double *Output = NULL;

int main(int argc, char *argv[]) {
  __int64 elapsedTime;
  __int64 startTime = GetTickCount();

  if(argc != 2) {
    cout << "1 arg is needed.\n";
    return 1;
  }
  int divide = atoi(argv[1]);
  double area = 0.0;

  Input = (double*)malloc(SizeInBytes);
  CHECK_NOT_NULL(Input);
  Output = (double*)malloc(SizeInBytes);
  CHECK_NOT_NULL(Output);

  const double range = 
    (Max - Min) / (double)(Width - 1);
  double midValue = Min + range;
  Input[0] = Min;
  Input[Width - 1] = Max;
  for(int i = 1; i < Width - 1; i++) {
    Input[i] = midValue;
    midValue += range;
  }
  //PrintArray("Input", Input, Width);

  double *dev_input;
  double *dev_output;
  CHECK_ERROR(cudaMalloc((void**)&dev_input, 
    SizeInBytes));
  CHECK_ERROR(cudaMalloc((void**)&dev_output, 
    SizeInBytes));
  CHECK_ERROR(cudaMemcpy(dev_input, Input, 
    SizeInBytes, cudaMemcpyHostToDevice));
  
  GetArea<<<BlockAndThread, BlockAndThread>>>(
    dev_input, dev_output, Width, divide);
  
  CHECK_ERROR(cudaMemcpy(Output, dev_output, 
    SizeInBytes, cudaMemcpyDeviceToHost));
  //PrintArray("Output", Output, Width);
  
  for(unsigned int i = 1; i < Width; i++) 
    area += Output[i];
  CHECK_ERROR(cudaFree(dev_input));
  CHECK_ERROR(cudaFree(dev_output));
  free(Input);
  free(Output);
  elapsedTime = GetTickCount() - startTime;
  cout.setf(ios::fixed);
  cout.width(14);
  cout << (__int64)divide * Width << '\t' <<
    area << '\t' << elapsedTime << endl;
  return 0;
}

/*
void PrintArray(const string arrayName, 
  const double *arrayData, const unsigned int length) {
  
  cout << arrayName.c_str() << ":" << endl;
  for(int i = 0; i < length; i++) 
    cout << arrayData[i] << " ";
  cout << endl << endl;
}
*/
